import { system, world, BlockPermutation, ItemStack } from "@minecraft/server";
function setDroneToBlock(drone) {
    let bottomBlock = drone.dimension.getBlockFromRay(drone.location, {x: 0, y: -1, z: 0}).block;
    drone.getTags().forEach(tag => {
        let itemToDrop;
        if (tag === "tnt") {
            itemToDrop = "tnt";
        } else if (tag === "fire") {
            itemToDrop = "flint_and_steel";
        } else if (tag === "dispenser") {
            itemToDrop = "dispenser";
        };
        if (itemToDrop) {
            drone.dimension.spawnItem(new ItemStack("minecraft:" + itemToDrop, 1), drone.location);
        }
    });
    drone.applyImpulse({x: 0, y: -10, z: 0});
    drone.playAnimation('animation.copper_drone.landing');
    system.runTimeout(() => {
        if (bottomBlock.above().isAir) {
            bottomBlock.above().setPermutation(BlockPermutation.resolve('cd:copper_drone_block'));
        } else {
            drone.dimension.getBlock(drone.location).setPermutation(BlockPermutation.resolve('cd:copper_drone_block'));
        }
        drone.remove();
    }, 13);
}
function setDroneToSpy(drone, player, tameable) {
    if (player.cameraViewing) return;
    const radius = 7;
    drone.triggerEvent('cd:untame');
    player.cameraViewing = system.runInterval(() => {
        if (player.isSneaking || !drone.isValid() || player.getComponent("minecraft:inventory")?.container?.getItem(player?.selectedSlotIndex).typeId !== "cd:drone_controller_spy_mode") {
            if (drone.isValid() && player.getComponent("minecraft:inventory")?.container?.getItem(player?.selectedSlotIndex).typeId === "cd:drone_controller_spy_mode") {
                if (drone?.getTags()?.includes("tnt")) {
                    drone.dimension.createExplosion(drone.location, 8, {causesFire: true, breakBlocks: true, source: drone});
                    drone.remove();
                };
            };
            system.clearRun(player.cameraViewing);
            player.cameraViewing = undefined;
            system.runTimeout(() => {
                player.camera.clear();
                drone.isValid() ? tameable?.tame(player): undefined;
            }, 20);
            return;
        };
        let message;
        if (drone?.getTags()?.includes("tnt")) {
            message = `Sneak to §r§c§lExplode`;
        } else {
            message = `Sneak to §r§a§lReturn`;
        }
        player.onScreenDisplay.setActionBar(message);
        const { x: xDrone, y: yDrone, z: zDrone } = drone.location;
        const { x: xView, y: yView, z: zView } = player.getViewDirection();
        let newYView = Math.min(Math.max(yView, -0.5), 0.5);
        player.camera.setCamera("minecraft:free", {
            easeOptions: { easeTime: 0.2, easeType: 'Linear' },
            location: {
                x: xDrone + xView * radius,
                y: 2 + yDrone + (yView * radius)+2,
                z: zDrone + zView * radius
            },
            facingEntity: drone
        });
        let nextPos = player.isJumping ? drone.location : {x: xDrone - 0.75 * xView, y: yDrone - 0.6 * newYView, z: zDrone - 0.75 * zView};
        drone.tryTeleport(nextPos, {checkForBlocks: true});
    })
}
world.afterEvents.entityHurt.subscribe((e) => {
    let drone = e.damageSource.damagingEntity;
    if (drone?.typeId === "cd:copper_drone" && drone?.getTags()?.includes("fire")) {
        e.hurtEntity.setOnFire(10);
    }
});

world.beforeEvents.worldInitialize.subscribe(e => {
    e.itemComponentRegistry.registerCustomComponent('cd:controller', {
        onUse: (e) => {
            let player = e.source;
            let blockLooking = player.getBlockFromViewDirection({maxDistance: 32})?.block;
            if (blockLooking?.typeId === "cd:broken_copper_drone_block") {
                player.onScreenDisplay.setActionBar('§c§lThis drone is broken!');
                return;
            }
            let droneLooking = player.getEntitiesFromViewDirection({maxDistance: 32, type: 'cd:copper_drone', tags: [player.name]})[0]?.entity;
            let holdItem = e.itemStack;
            let coolDownComponent = holdItem.getComponent('minecraft:cooldown')
            let isCoolDown = coolDownComponent?.getCooldownTicksRemaining(player) > 18;
            if (holdItem?.typeId.startsWith('cd:drone_controller_')) {
                let currentState = holdItem.typeId.replace('cd:drone_controller_', '');
                if (typeof coolDownComponent !== 'undefined' && !isCoolDown && currentState === 'control_mode') return;
                let nextState;
                if (blockLooking?.typeId === "cd:copper_drone_block" && currentState === 'control_mode') {
                    blockLooking.setPermutation(BlockPermutation.resolve('minecraft:air'));
                    let newDrone = blockLooking.dimension.spawnEntity("cd:copper_drone", blockLooking.center());
                    newDrone.getComponent("minecraft:tameable").tame(player);
                    newDrone.addTag(player.name);
                    newDrone.addEffect('regeneration', 160, {amplifier: 255, showParticles: false});
                    nextState = 'control_mode';
                };
                if (droneLooking && currentState === 'control_mode' && holdItem.getComponent('minecraft:cooldown')?.getCooldownTicksRemaining(player) > 18) {
                    setDroneToBlock(droneLooking);
                    nextState = 'control_mode';
                };
                if (currentState === 'spy_mode' && droneLooking) {
                    setDroneToSpy(droneLooking, player, droneLooking.getComponent("minecraft:tameable"));
                    nextState = 'spy_mode';
                }
                if (currentState === 'call_mode') {
                    if (droneLooking) {
                        droneLooking.teleport(player.location);
                        nextState = 'call_mode';
                    } else if (player.isSneaking) {
                        player.dimension.getEntities({type: "cd:copper_drone", tags:[player.name]}).forEach((drone) => {drone.teleport(player.location)});
                        nextState = 'call_mode';
                    }
                };
                if (!nextState) {
                    const states = ['control_mode', 'spy_mode', 'call_mode'];
                    const currentIndex = states.indexOf(currentState);
                    nextState = states[(currentIndex + 1) % 3];
                }
                player.getComponent("minecraft:inventory")?.container?.setItem(player.selectedSlotIndex, new ItemStack(`cd:drone_controller_${nextState}`, 1));
            };
        },
    });
});
world.afterEvents.entityDie.subscribe((e) => {
    let drone = e.deadEntity;
    if (drone.typeId === "cd:copper_drone" && e.damageSource.cause !== "selfDestruct") {
        let bottomBlock = drone.dimension.getBlockFromRay(drone.location, {x: 0, y: -1, z: 0}).block;
        if (bottomBlock.above().isAir) {
            bottomBlock.above().setPermutation(BlockPermutation.resolve('cd:broken_copper_drone_block'));
        } else {
            drone.dimension.getBlock(drone.location).setPermutation(BlockPermutation.resolve('cd:broken_copper_drone_block'));
        }
    }
});
world.afterEvents.projectileHitEntity.subscribe((e) => {
    if (e.source?.typeId === "cd:copper_drone") {
        let hitEntity = e.getEntityHit().entity;
        hitEntity?.dimension.getEntities({location: hitEntity.location, maxDistance: 4, excludeFamilies: ["player"], excludeTypes: ["minecraft:item", "minecraft:xp_orb"]}).forEach(entity => {
            entity.applyImpulse({x:e.hitVector.x * 0.5, y: 1, z:e.hitVector.z * 0.5});
            entity.applyDamage(10);
        });;
    }
})
world.afterEvents.projectileHitBlock.subscribe((e) => {
    if (e.source?.typeId === "cd:copper_drone") {
        e.dimension.getEntities({location: e.location, maxDistance: 4, excludeFamilies: ["player"], excludeTypes: ["minecraft:item", "minecraft:xp_orb"]}).forEach(entity => {
            entity.applyImpulse({x:e.hitVector.x * 0.5, y: 1, z:e.hitVector.z * 0.5});
            entity.applyDamage(10)
        });;
    }  
})
function usedReduce(targetUseItem) {
    if (targetUseItem.getGameMode() !== "survival") {return;}
    let currentItem = targetUseItem.getComponent("minecraft:equippable").getEquipment("Mainhand");
    let afterItem;
    if (currentItem.amount > 1) {
        afterItem = new ItemStack(currentItem.typeId, currentItem.amount - 1);
    } else {
        afterItem = undefined;
    }
    targetUseItem.getComponent("minecraft:equippable").getEquipmentSlot('Mainhand').setItem(afterItem)
}
world.afterEvents.playerInteractWithEntity.subscribe((e) => {
    let drone = e.target;
    let player = e.player;
    let item = e.itemStack?.typeId.replace('minecraft:', '').replace('cd:drone_controller_', '');
    let tameable = drone.getComponent("minecraft:tameable");
    if (drone.typeId === "cd:copper_drone" && tameable?.isTamed === true && drone?.getTags()?.includes(player.name)) {
        switch (item) {
            case "control_mode":
                if (e.itemStack.getComponent('minecraft:cooldown')?.getCooldownTicksRemaining(player) === 0) {
                    let coolDownRemote = new ItemStack('cd:drone_controller_control_mode', 1);
                    coolDownRemote.getComponent('minecraft:cooldown').startCooldown(player);
                    player.getComponent("minecraft:inventory")?.container?.setItem(player.selectedSlotIndex, coolDownRemote);
                    setDroneToBlock(drone)
                };
            break;
            case "spy_mode":
                setDroneToSpy(drone, player, tameable);
            break;
            case "tnt": 
                drone.addTag("tnt");
            break;
            case "dispenser": 
                drone.addTag("dispenser");
            break;
            case "redstone_block":
                drone.addEffect('strength', 1200, {amplifier: 4, showParticles: false});
                drone.setProperty('cd:is_reinforced', true);
                usedReduce(player);
                system.runTimeout(() => {
                    drone.isValid() ? drone.setProperty('cd:is_reinforced', false) : undefined;
                }, 1200)
            break;
            case "iron_block":
                drone.triggerEvent('cd:armor_mode');
                drone.addEffect('regeneration', 600, {amplifieo: 255, showParticles: false});
                usedReduce(player);
            break;
            case "flint_and_steel":
                drone.addTag("fire");
            break;
        };
    };
})